@@ -1,8 +1,8 @@
 ;; -*- no-byte-compile: t; lexical-binding: nil -*-
-(define-package "outline-indent" "1.1.7"
+(define-package "outline-indent" "20250921.2109"
   "Folding text based on indentation (origami alternative)."
   '((emacs "26.1"))
   :url "https://github.com/jamescherti/outline-indent.el"
-  :commit "75e1b5d41e7b18ce1c04ec33a20af2fbed5baa96"
-  :revdesc "1.1.7-0-g75e1b5d41e7b"
+  :commit "c99cf257ab5c34cb894bdd6238be9dda9fce9e85"
+  :revdesc "c99cf257ab5c"
   :keywords '("outlines"))
