@@ -1,10 +1,10 @@
 ;;; outline-indent.el --- Folding text based on indentation (origami alternative) -*- lexical-binding: t; -*-
 
-;; Copyright (C) 2024-2026 James Cherti | https://www.jamescherti.com/contact/
+;; Copyright (C) 2024-2025 James Cherti | https://www.jamescherti.com/contact/
 
 ;; Author: James Cherti
-;; Package-Version: 1.1.7
-;; Package-Revision: 1.1.7-0-g75e1b5d41e7b
+;; Package-Version: 20250921.2109
+;; Package-Revision: c99cf257ab5c
 ;; URL: https://github.com/jamescherti/outline-indent.el
 ;; Keywords: outlines
 ;; Package-Requires: ((emacs "26.1"))
@@ -49,9 +49,8 @@
 ;; ------------------------
 ;; (use-package outline-indent
 ;;   :ensure t
-;;   :commands outline-indent-minor-mode
 ;;   :custom
-;;   (outline-indent-ellipsis " ▼"))
+;;   (outline-indent-ellipsis " ▼ "))
 ;;
 ;; Activation:
 ;; -----------
@@ -90,24 +89,18 @@
   "Default indentation offset.
 If nil, the offset is automatically determined based on the current mode.
 This value is used to calculate outline levels from the current indentation."
-  :type '(choice (natnum :tag "Custom width")
-                 (natnum :tag "Auto detect" nil))
+  :type '(choice (integer :tag "Custom width")
+                 (const :tag "Auto detect" nil))
   :group 'outline-indent)
 
 (defcustom outline-indent-shift-width nil
   "Shift width used for indentation adjustments during promotion and demotion.
 If nil, this value defaults to `outline-indent-default-offset', which is
 automatically determined according to the current mode."
-  :type '(choice (natnum :tag "Custom width")
+  :type '(choice (integer :tag "Custom width")
                  (const :tag "Auto detect" nil))
   :group 'outline-indent)
 
-(defcustom outline-indent-maximum-level nil
-  "Maximum allowed depth for outline indentation.
-If nil, there is no maximum."
-  :type '(choice (natnum :tag "Maximum level")
-                 (const :tag "No limit" nil)))
-
 (defcustom outline-indent-ellipsis nil
   "String used as the ellipsis character in `outline-indent-mode'.
 
@@ -133,10 +126,11 @@ suitable for representing folded content."
   :type '(choice string (const nil))
   :group 'outline-indent)
 
+
 (define-obsolete-variable-alias
   'outline-indent-make-window-start-visible
   'make-window-start-visible
-  "1.1.4"
+  "1.1.2"
   "Obsolete. Use `make-window-start-visible' instead.")
 
 (defcustom outline-indent-insert-heading-add-blank-line nil
@@ -237,12 +231,6 @@ It is recommended to keep this set to t for improved behavior."
     map)
   "Keymap for `outline-indent-minor-mode'.")
 
-(defvar outline-indent-conflicting-modes
-  '(hs-minor-mode origami-mode yafolding-mode)
-  "List of minor modes that may conflict with `outline-indent-minor-mode'.
-These modes will be automatically disabled when `outline-indent-minor-mode' is
-enabled.")
-
 ;;; Internal variables
 
 (defconst outline-indent--has-ts-modes
@@ -251,69 +239,66 @@ enabled.")
   "Non-nil if Emacs supports tree-sitter modes (Emacs 29.1+).")
 
 (defvar outline-indent--mode-basic-offset-map
-  (list
-   (cons 'typescript-ts-base-mode 'typescript-ts-mode-indent-offset)
-   (cons 'ada-ts-mode 'ada-ts-mode-indent-offset)
-   (cons 'sh-mode 'sh-basic-offset)
-   (cons 'c++-mode 'c-basic-offset)
-   (cons 'c-mode 'c-basic-offset)
-   (cons 'cmake-mode 'cmake-tab-width)
-   (cons 'cperl-mode 'cperl-indent-level)
-   (cons 'crystal-mode 'crystal-indent-level)
-   (cons 'css-mode 'css-indent-offset)
-   (cons 'd-mode 'c-basic-offset)
-   (cons 'default 'standard-indent)
-   (cons 'enh-ruby-mode 'enh-ruby-indent-level)
-   (cons 'erlang-mode 'erlang-indent-level)
-   (cons 'java-mode 'c-basic-offset)
-   (cons 'jde-mode 'c-basic-offset)
-   (cons 'js-mode 'js-indent-level)
-   (cons 'js2-mode 'js2-basic-offset)
-   (cons 'js3-mode 'js3-indent-level)
-   (cons 'json-mode 'js-indent-level)
-   (cons 'lua-mode 'lua-indent-level)
-   (cons 'nxml-mode 'nxml-child-indent)
-   (cons 'objc-mode 'c-basic-offset)
-   (cons 'pascal-mode 'pascal-indent-level)
-   (cons 'perl-mode 'perl-indent-level)
-   (cons 'php-mode 'c-basic-offset)
-   (cons 'plantuml-mode 'plantuml-indent-level)
-   (cons 'protobuf-mode 'c-basic-offset)
-   (cons 'pug-mode 'pug-tab-width)
-   (cons 'raku-mode 'raku-indent-offset)
-   (cons 'ruby-mode 'ruby-indent-level)
-   (cons 'rust-mode 'rust-indent-offset)
-   (cons 'rustic-mode 'rustic-indent-offset)
-   (cons 'scala-mode 'scala-indent:step)
-   (cons 'sgml-mode 'sgml-basic-offset)
-   (cons 'swift-mode 'swift-mode:basic-offset)
-   (cons 'typescript-mode 'typescript-indent-level)
-   (cons 'ada-mode 'ada-indent)
-   (cons 'bash-ts-mode 'sh-basic-offset)
-   (cons 'js-json-mode 'js-indent-level)
-   (cons 'c++-ts-mode 'c-ts-mode-indent-offset)
-   (cons 'c-ts-mode 'c-ts-mode-indent-offset)
-   (cons 'cmake-ts-mode 'cmake-ts-mode-indent-offset)
-   (cons 'go-ts-mode 'go-ts-mode-indent-offset)
-   (cons 'gpr-ts-mode 'gpr-ts-mode-indent-offset)
-   (cons 'java-ts-mode 'java-ts-mode-indent-offset)
-   (cons 'js-ts-mode 'js-indent-level)
-   (cons 'json-ts-mode 'json-ts-mode-indent-offset)
-   (cons 'rust-ts-mode 'rust-ts-mode-indent-offset)
-   (cons 'yaml-ts-mode '(yaml-indent-offset 2))
-   (cons 'python-mode 'python-indent-offset)
-   (cons 'python-ts-mode 'python-indent-offset)
-   (cons 'ursa-ts-mode 'ursa-ts-mode-indent-offset)
-   (cons 'vhdl-mode 'vhdl-basic-offset)
-   (cons 'xquery-mode 'xquery-mode-indent-width)
-   (cons 'groovy-mode '(groovy-indent-offset
-                        tab-width))
-   (cons 'yaml-mode '(yaml-indent-offset
-                      tab-width))
-   (cons 'web-mode '(web-mode-markup-indent-offset
+  '((typescript-ts-base-mode typescript-ts-mode-indent-offset)
+    (ada-ts-mode     ada-ts-mode-indent-offset)
+    (sh-mode         sh-basic-offset)
+    (c++-mode        c-basic-offset)
+    (c-mode          c-basic-offset)
+    (cmake-mode      cmake-tab-width)
+    (cperl-mode      cperl-indent-level)
+    (crystal-mode    crystal-indent-level)
+    (css-mode        css-indent-offset)
+    (d-mode          c-basic-offset)
+    (default         standard-indent)
+    (enh-ruby-mode   enh-ruby-indent-level)
+    (erlang-mode     erlang-indent-level)
+    (java-mode       c-basic-offset)
+    (jde-mode        c-basic-offset)
+    (js-mode         js-indent-level)
+    (js2-mode        js2-basic-offset)
+    (js3-mode        js3-indent-level)
+    (json-mode       js-indent-level)
+    (lua-mode        lua-indent-level)
+    (nxml-mode       nxml-child-indent)
+    (objc-mode       c-basic-offset)
+    (pascal-mode     pascal-indent-level)
+    (perl-mode       perl-indent-level)
+    (php-mode        c-basic-offset)
+    (plantuml-mode   plantuml-indent-level)
+    (protobuf-mode   c-basic-offset)
+    (pug-mode        pug-tab-width)
+    (raku-mode       raku-indent-offset)
+    (ruby-mode       ruby-indent-level)
+    (rust-mode       rust-indent-offset)
+    (rustic-mode     rustic-indent-offset)
+    (scala-mode      scala-indent:step)
+    (sgml-mode       sgml-basic-offset)
+    (swift-mode      swift-mode:basic-offset)
+    (typescript-mode typescript-indent-level)
+    (ada-mode        ada-indent)
+    (bash-ts-mode    sh-basic-offset)
+    (js-json-mode    js-indent-level)
+    (c++-ts-mode     c-ts-mode-indent-offset)
+    (c-ts-mode       c-ts-mode-indent-offset)
+    (cmake-ts-mode   cmake-ts-mode-indent-offset)
+    (go-ts-mode      go-ts-mode-indent-offset)
+    (gpr-ts-mode     gpr-ts-mode-indent-offset)
+    (java-ts-mode    java-ts-mode-indent-offset)
+    (js-ts-mode      js-indent-level)
+    (json-ts-mode    json-ts-mode-indent-offset)
+    (rust-ts-mode    rust-ts-mode-indent-offset)
+    (yaml-ts-mode    yaml-indent-offset 2)
+    (python-mode     python-indent-offset)
+    (python-ts-mode  python-indent-offset)
+    (ursa-ts-mode    ursa-ts-mode-indent-offset)
+    (vhdl-mode       vhdl-basic-offset)
+    (xquery-mode     xquery-mode-indent-width)
+    (groovy-mode     groovy-indent-offset tab-width)
+    (yaml-mode       yaml-indent-offset tab-width)
+    (web-mode        web-mode-markup-indent-offset
                      web-mode-code-indent-offset
                      web-mode-sql-indent-offset
-                     web-mode-css-indent-offset)))
+                     web-mode-css-indent-offset))
   "A mapping from hook variables to language types.")
 
 ;;; Internal Functions
@@ -323,8 +308,8 @@ enabled.")
 Searches `outline-indent--mode-basic-offset-map', falling back to `default'."
   (when mode
     (or (assoc mode outline-indent--mode-basic-offset-map)
-        (when-let* ((parent-mode (get mode 'derived-mode-parent)))
-          (outline-indent--get-mode-basic-offset-entry parent-mode))
+        (outline-indent--get-mode-basic-offset-entry
+         (get mode 'derived-mode-parent))
         (assoc 'default outline-indent--mode-basic-offset-map))))
 
 (defun outline-indent--get-mode-basic-offset (mode)
@@ -336,9 +321,6 @@ is found."
     (let ((list-vars (cdr entry))
           result)
       (catch 'done
-        (when (symbolp list-vars)
-          (setq list-vars (list list-vars)))
-
         (dolist (var list-vars)
           (cond
            ((numberp var)
@@ -370,41 +352,13 @@ follow the mode-specific coding style automatically."
       (unless outline-indent-shift-width
         (setq-local outline-indent-shift-width major-mode-offset)))))
 
-(defun outline-indent-level-ignore-empty-lines ()
-  "Determine the outline level based on the current indentation."
-  (let* ((indentation-string (match-string 1))
-         (indentation-width (if indentation-string
-                                (string-width indentation-string)
-                              0))
-         (depth (if (and (= indentation-width 0)
-                         (save-excursion
-                           (beginning-of-line)
-                           (looking-at-p "^\\s-*$")))
-                    0
-                  (+ 1 (/ indentation-width
-                          (max (if outline-indent-default-offset
-                                   outline-indent-default-offset
-                                 1)
-                               1))))))
-    (if outline-indent-maximum-level
-        (min depth (1+ outline-indent-maximum-level))
-      depth)))
-
 (defun outline-indent-level ()
   "Determine the outline level based on the current indentation."
-  (let* ((indentation-width (current-indentation))
-         ;; TODO fix this
-         ;; (indentation-string (match-string 1))
-         ;; (indentation-width (if indentation-string
-         ;;                        (string-width indentation-string)
-         ;;                      0))
-         (depth (1+ (/ indentation-width
-                       (max (or outline-indent-default-offset
-                                1)
-                            1)))))
-    (if outline-indent-maximum-level
-        (min depth (1+ outline-indent-maximum-level))
-      depth)))
+  (+ 1 (/ (current-indentation)
+          (max (if outline-indent-default-offset
+                   outline-indent-default-offset
+                 1)
+               1))))
 
 (defun outline-indent--update-ellipsis ()
   "Update the buffer's outline ellipsis."
@@ -473,11 +427,7 @@ to insert content at the same indentation level after the current fold."
       (when outline-indent-insert-heading-add-blank-line
         (newline)
         (forward-line -1))
-      (indent-to initial-indentation)))
-
-  (when (and (bound-and-true-p evil-local-mode)
-             (fboundp 'evil-insert-state))
-    (funcall 'evil-insert-state)))
+      (indent-to initial-indentation))))
 
 (defun outline-indent-move-subtree-up (&optional arg)
   "Move the current subtree up past ARG headlines of the same level.
@@ -490,14 +440,66 @@ addressing the issue where the cursor might be reset after the operation."
   (outline-indent-move-subtree-down (- arg)))
 
 (defun outline-indent--deactivate-region ()
-  "Deactivate the current region and relocate point to the start.
-If a region is currently active, moves the point to the beginning of the region
-and disables the mark. This ensures a consistent starting state for subsequent
-selection operations."
+  "Deactivate the current region and move point to the start of the region."
   (when (use-region-p)
     (goto-char (region-beginning))
     (deactivate-mark)))
 
+(defun outline-indent-shift-right (&optional _which arg)
+  "Increasing the indentation level.
+The global variable `outline-indent-shift-width' or
+`outline-indent-default-offset' is used to determine the number of spaces to
+indent the subtree.
+WHICH is ignored (backward compatibility with `outline-demote').
+If ARG is positive, indent the outline. If ARG is negative, unindent the
+outline. Defaults to 1 if ARG is nil."
+  (interactive)
+  (unless arg
+    (setq arg 1))
+  (outline-indent--deactivate-region)
+  (let ((shift-right (>= arg 0))
+        (column (current-column))
+        (shift-width
+         (cond (outline-indent-shift-width
+                (max outline-indent-shift-width 1))
+
+               (t
+                (max outline-indent-default-offset 1)))))
+    (let ((folded (save-match-data
+                    (outline-end-of-heading)
+                    (outline-invisible-p))))
+      (save-excursion
+        (outline-back-to-heading)
+        (let ((start (point))
+              (end (save-excursion
+                     (outline-end-of-subtree)
+                     (point))))
+          (indent-rigidly start end (if shift-right
+                                        shift-width
+                                      (* -1 shift-width)))))
+      (if shift-right
+          (move-to-column (+ column shift-width))
+        (move-to-column (max (- column shift-width) 0)))
+      (if folded
+          (outline-hide-subtree)))))
+
+(defun outline-indent-shift-left (&optional _which)
+  "Decreasing the indentation level. Equivalent to `outline-promote'.
+The global variable `outline-indent-shift-width' or
+`outline-indent-default-offset' is used to determine the number of spaces to
+unindent the subtree.
+WHICH is ignored (backward compatibility with `outline-promote')."
+  (interactive)
+  (outline-indent-shift-right nil -1))
+
+(defalias 'outline-indent-demote #'outline-indent-shift-right
+  "Deprecated alias for `outline-indent-shift-right'.")
+(make-obsolete 'outline-indent-demote 'outline-indent-shift-right "1.1.1")
+
+(defalias 'outline-indent-promote #'outline-indent-shift-left
+  "Deprecated alias for `outline-indent-shift-left'.")
+(make-obsolete 'outline-indent-promote 'outline-indent-shift-left "1.1.1")
+
 (defun outline-indent--advice-promote (orig-fun &rest args)
   "Advice function for `outline-indent-shift-left'.
 If `outline-indent-minor-mode' is active, use `outline-indent-insert-heading'.
@@ -692,22 +694,54 @@ ORIG-FUN is the original function being advised, and ARGS are its arguments."
 
 ;;; Functions
 
-(defun outline-indent-folded-p ()
-  "Return non-nil when the current heading is folded."
-  (save-excursion
-    (outline-back-to-heading)
-    (outline-end-of-heading)
-    (outline-invisible-p (point))))
+(defun outline-indent-backward-same-level (&optional arg)
+  "Move backward to the ARG'th subheading at same indentation level as this one.
+Stop at the first and last indented blocks of a superior indentation."
+  (interactive "p")
+  (unless arg
+    (setq arg 1))
+  (if (advice-member-p 'outline-indent--advice-backward-same-level
+                       'outline-backward-same-level)
+      (outline-backward-same-level arg)
+    (outline-indent--advice-backward-same-level 'outline-backward-same-level
+                                                arg)))
 
-(defun outline-indent--legacy-outline-hide-subtree (&optional event)
-  "Hide everything after this heading at deeper levels.
-If non-nil, EVENT should be a mouse event."
-  (interactive (list last-nonmenu-event))
-  (save-excursion
-    (when (mouse-event-p event)
-      (mouse-set-point event))
-    (outline-flag-subtree t)))
+(defun outline-indent-forward-same-level (&optional arg)
+  "Move forward to the ARG'th subheading at same indentation level as this one.
+Stop at the first and last indented blocks of a superior indentation."
+  (interactive "p")
+  (unless arg
+    (setq arg 1))
+  (if (advice-member-p 'outline-indent--advice-forward-same-level
+                       'outline-forward-same-level)
+      (outline-forward-same-level arg)
+    (outline-indent--advice-forward-same-level 'outline-forward-same-level
+                                               arg)))
 
+(defun outline-indent-select ()
+  "Select the indented block at point."
+  (interactive)
+  (outline-indent--deactivate-region)
+  (let ((begin (save-excursion
+                 (outline-back-to-heading)
+                 (point)))
+        (end (outline-indent--next-lower-or-equal-indentation)))
+    (goto-char (+ end 1))
+    (push-mark)
+    (goto-char begin)
+    (activate-mark)))
+
+(defun outline-indent-close-folds ()
+  "Close all folds."
+  (interactive)
+  (with-no-warnings (outline-hide-sublevels 1)))
+
+(defun outline-indent-open-folds ()
+  "Open all folds."
+  (interactive)
+  (outline-show-all))
+
+;; TODO: Use the original one
 (defun outline-indent--legacy-outline-show-entry ()
   "Show the body directly following this heading. (Emacs version.)
 Show the heading too, if it is currently invisible."
@@ -722,177 +756,58 @@ Show the heading too, if it is currently invisible."
                              (point)))
                          nil)))
 
-(defun outline-indent-close-level (level)
-  "Close the folds at the level: LEVEL."
-  (outline-hide-sublevels level))
-
-;;; Interactive functions
-
-;;;###autoload
-(defun outline-indent-backward-same-level (&optional arg)
-  "Move the cursor to the previous heading that is at the same indentation level.
-Move backward to the ARG'th subheading at same indentation level as this one.
-Stop at the first and last indented blocks of a superior indentation."
-  (interactive "p")
-  (unless arg
-    (setq arg 1))
-  (if (advice-member-p 'outline-indent--advice-backward-same-level
-                       'outline-backward-same-level)
-      (outline-backward-same-level arg)
-    (outline-indent--advice-backward-same-level 'outline-backward-same-level
-                                                arg)))
-
-;;;###autoload
-(defun outline-indent-forward-same-level (&optional arg)
-  "Move the cursor to the next heading that is at the same indentation level.
-Move forward to the ARG'th subheading at same indentation level as this one.
-Stop at the first and last indented blocks of a superior indentation."
-  (interactive "p")
-  (unless arg
-    (setq arg 1))
-  (if (advice-member-p 'outline-indent--advice-forward-same-level
-                       'outline-forward-same-level)
-      (outline-forward-same-level arg)
-    (outline-indent--advice-forward-same-level 'outline-forward-same-level
-                                               arg)))
-
-;;;###autoload
-(defun outline-indent-select ()
-  "Select the indented block at point.
-Identifies the heading and all associated indented content, then activates a
-visual region spanning from the heading start to the end of the block."
-  (interactive)
-  (outline-indent--deactivate-region)
-  (let ((begin (save-excursion
-                 (outline-back-to-heading)
-                 (point)))
-        (end (outline-indent--next-lower-or-equal-indentation)))
-    (push-mark (1+ end) nil t)
-    (goto-char begin)
-    (activate-mark)))
-
-;;;###autoload
-(defun outline-indent-close-folds ()
-  "Close all folds and ensure the first heading remains visible."
-  (interactive)
-  (with-no-warnings
-    (outline-hide-sublevels 1))
-
-  (let ((heading-point (save-excursion
-                         (condition-case nil
-                             (progn
-                               (goto-char (window-start))
-                               (beginning-of-visual-line)
-                               (if (outline-on-heading-p)
-                                   (point)
-                                 (outline-back-to-heading t)))
-                           (error
-                            nil)))))
-    ;; Ensure folded headings remain visible after hiding subtrees. Fixes a
-    ;; bug in outline and Evil where headings could scroll out of view when
-    ;; their subtrees were folded. TODO Send a patch to Emacs and/or Evil
-    (when (and heading-point
-               (< heading-point (window-start)))
-      (set-window-start (selected-window) heading-point t))))
-
-;;;###autoload
-(defun outline-indent-open-folds ()
-  "Open all folds."
-  (interactive)
-  (outline-show-all))
-
-;;;###autoload
 (defun outline-indent-open-fold ()
   "Open fold at point."
   (interactive)
-  ;; Workaround for an outline-mode limitation: when jumping via imenu or
-  ;; search, sibling headings above the current one and at the same level
-  ;; often remain hidden. This ensures all sub-items at the current level are
-  ;; revealed, preventing the 'isolated item' effect.
-  (save-excursion
-    ;; Climbing as long as a parent heading exists
-    (catch 'done
-      (while (not (bobp))
-        (condition-case nil
-            (progn (outline-up-heading 1 t))
-          (error
-           (throw 'done t)))
-
-        (outline-show-children))))
-
-  ;; Repeatedly reveal children and body until the entry is no longer folded
   (condition-case nil
-      (progn
-        (let* ((on-invisible-heading (when (outline-on-heading-p t)
-                                       (outline-invisible-p)))
-               (on-visible-heading (save-excursion
-                                     (beginning-of-line)
-                                     (outline-on-heading-p))))
-          (when (use-region-p)
-            (outline-indent-select))
+      (let ((on-visible-heading (when (outline-on-heading-p t)
+                                  (outline-invisible-p))))
+        (save-excursion
+          (while (outline-indent-folded-p)
+            ;; Repeatedly reveal children and body until the entry is no
+            ;; longer folded
+            (save-excursion
+              (outline-back-to-heading)
+              (outline-show-children)
+              (outline-indent--legacy-outline-show-entry))))
 
-          (save-excursion
-            (while (outline-indent-folded-p)
-              ;; Repeatedly reveal children and body until the entry is no
-              ;; longer folded
-              (save-excursion
-                (outline-back-to-heading)
-                (outline-show-children)
-                (outline-indent--legacy-outline-show-entry))))
-
-          ;; If the header was previously hidden, hide the subtree to collapse
-          ;; it. Otherwise, leave the fold open. This allows the user to decide
-          ;; whether to expand the content under the cursor.
-          (when (and on-invisible-heading (not on-visible-heading))
-            (outline-indent--legacy-outline-hide-subtree))))
+        (when on-visible-heading
+          (outline-indent--legacy-outline-hide-subtree)))
     ;; Ignore `outline-before-first-heading'
     (outline-before-first-heading
      nil)))
 
-;;;###autoload
+;; TODO: Use the original one
+(defun outline-indent--legacy-outline-hide-subtree (&optional event)
+  "Hide everything after this heading at deeper levels.
+If non-nil, EVENT should be a mouse event."
+  (interactive (list last-nonmenu-event))
+  (save-excursion
+    (when (mouse-event-p event)
+      (mouse-set-point event))
+    (outline-flag-subtree t)))
+
 (defun outline-indent-close-fold ()
-  "Close the current heading's subtree in a robust manner.
-
-If the current heading is folded or contains no content, move to the previous
-heading with a higher level and close its subtree.
-
-Otherwise, close the current subtree. Ensures that folded headings remain
-visible in the window after hiding."
+  "Close fold at point."
   (interactive)
   (condition-case nil
       (save-excursion
-        ;; Move to the current heading; error if before the first heading
         (outline-back-to-heading)
-
-        (let ((heading-point (point)))
-          ;; If the current heading is folded, or if it contains no content,
-          ;; move to the previous higher-level heading.
-          (when (or (outline-indent-folded-p)  ; Folded?
-                    ;; Fold without any content
-                    (let ((start (save-excursion (end-of-line)
-                                                 (point)))
-                          (end (save-excursion (outline-end-of-subtree)
-                                               (point))))
-                      (= start end)))
-            ;; Try to move up to previous higher-level heading
-            (outline-up-heading 1 t)
-            (setq heading-point (point)))
-
-          (when (outline-on-heading-p)
-            (outline-indent--legacy-outline-hide-subtree))
-
-          ;; Ensure folded headings remain visible after hiding subtrees. Fixes
-          ;; a bug in outline and Evil where headings could scroll out of view
-          ;; when their subtrees were folded.
-          ;; TODO Send a patch to Emacs and/or Evil
-          (when (and heading-point
-                     (< heading-point (window-start)))
-            (set-window-start (selected-window) heading-point t))))
+        (if (or (outline-indent-folded-p)  ; Folded?
+                ;; Fold without any content
+                (let ((start (save-excursion (end-of-line) (point)))
+                      (end (save-excursion (outline-end-of-subtree) (point))))
+                  (= start end)))
+            (progn
+              (when (eq (ignore-errors (outline-up-heading 1 t) :success)
+                        :success)
+                (when (outline-on-heading-p)
+                  (outline-indent--legacy-outline-hide-subtree))))
+          (outline-indent--legacy-outline-hide-subtree)))
     ;; Ignore `outline-before-first-heading'
     (outline-before-first-heading
      nil)))
 
-;;;###autoload
 (defun outline-indent-open-fold-rec ()
   "Open fold at point recursively."
   (interactive)
@@ -902,13 +817,22 @@ visible in the window after hiding."
     (outline-before-first-heading
      nil)))
 
-;;;###autoload
 (defun outline-indent-toggle-fold ()
   "Open or close a fold under point."
   (interactive)
   (outline-toggle-children))
 
-;;;###autoload
+(defun outline-indent-folded-p ()
+  "Return non-nil when the current heading is folded."
+  (save-excursion
+    (outline-back-to-heading)
+    (outline-end-of-heading)
+    (outline-invisible-p (point))))
+
+(defun outline-indent-close-level (level)
+  "Close the folds at the level: LEVEL."
+  (outline-hide-sublevels level))
+
 (defun outline-indent-toggle-level-at-point ()
   "Toggle the visibility of the indentation level under the cursor."
   (interactive)
@@ -924,90 +848,18 @@ visible in the window after hiding."
           (outline-hide-sublevels level)
         (outline-hide-sublevels (- level 1))))))
 
-;;;###autoload
-(defun outline-indent-shift-right (&optional _which arg)
-  "Increase the indentation level of the current indented block.
-The global variable `outline-indent-shift-width' or
-`outline-indent-default-offset' is used to determine the number of spaces to
-indent the subtree.
-WHICH is ignored (backward compatibility with `outline-demote').
-If ARG is positive, indent the outline. If ARG is negative, unindent the
-outline. Defaults to 1 if ARG is nil."
-  (interactive)
-  (unless arg
-    (setq arg 1))
-  (outline-indent--deactivate-region)
-  (let ((shift-right (>= arg 0))
-        (column (current-column))
-        (shift-width
-         (cond (outline-indent-shift-width
-                (max outline-indent-shift-width 1))
-
-               (t
-                (max outline-indent-default-offset 1)))))
-    (let ((folded (save-match-data
-                    (outline-end-of-heading)
-                    (outline-invisible-p))))
-      (save-excursion
-        (outline-back-to-heading)
-        (let ((start (point))
-              (end (save-excursion
-                     (outline-end-of-subtree)
-                     (point))))
-          (indent-rigidly start end (if shift-right
-                                        shift-width
-                                      (* -1 shift-width)))))
-      (if shift-right
-          (move-to-column (+ column shift-width))
-        (move-to-column (max (- column shift-width) 0)))
-      (if folded
-          (outline-hide-subtree)))))
-
-;;;###autoload
-(defun outline-indent-shift-left (&optional _which)
-  "Decrease the indentation level of the current indented block.
-The global variable `outline-indent-shift-width' or
-`outline-indent-default-offset' is used to determine the number of spaces to
-unindent the subtree.
-WHICH is ignored (backward compatibility with `outline-promote')."
-  (interactive)
-  (outline-indent-shift-right nil -1))
-
-(defalias 'outline-indent-demote #'outline-indent-shift-right
-  "Deprecated alias for `outline-indent-shift-right'.")
-(make-obsolete 'outline-indent-demote 'outline-indent-shift-right "1.1.1")
-
-(defalias 'outline-indent-promote #'outline-indent-shift-left
-  "Deprecated alias for `outline-indent-shift-left'.")
-(make-obsolete 'outline-indent-promote 'outline-indent-shift-left "1.1.1")
-
 ;;; Mode
 
-;; (defun outline-indent--heading-regexp ()
-;;   "Compute heading regexp based on leading indentation."
-;;   (rx-to-string
-;;    `(and line-start
-;;          (group (zero-or-more (any " \t")))
-;;          (not (any " \t\n")))))
-
 ;;;###autoload
 (define-minor-mode outline-indent-minor-mode
-  "Minor mode for folding text according to indentation levels."
+  "Toggle `outline-indent-minor-mode'.
+This mode sets up outline to work based on indentation."
   :lighter " OutlInd"
   :keymap outline-indent-minor-mode-map
   :group 'outline-indent
   (if outline-indent-minor-mode
       (progn
-        ;; Disable conflicting modes
-        (dolist (mode outline-indent-conflicting-modes)
-          (when (and (boundp mode)
-                     (symbol-value mode)
-                     (fboundp mode))
-            (save-excursion
-              (funcall mode -1))))
-
         (outline-indent--advise-func outline-indent-advise-outline-functions)
-
         ;; Enable minor mode
         (when (boundp 'outline-minor-mode-highlight)
           (setq-local outline-minor-mode-highlight nil))
@@ -1016,9 +868,8 @@ WHICH is ignored (backward compatibility with `outline-promote')."
         (setq-local outline-heading-alist nil)
         (setq-local outline-level #'outline-indent-level)
         (setq-local outline-heading-end-regexp "\n")
-        ;; (setq-local outline-regexp (outline-indent--heading-regexp))
-        (setq-local outline-regexp (rx line-start
-                                       (group (zero-or-more (any " \t")))
+        (setq-local outline-regexp (rx bol
+                                       (zero-or-more (any " \t"))
                                        (not (any " \t\n"))))
         (outline-indent--update-ellipsis)
         (outline-indent--setup-basic-offset)
